import datetime
import hashlib
import json
import logging
import urllib.parse
from copy import deepcopy
from typing import Set, List, Dict
from urllib.parse import urlparse
import re

import ckan.lib.dictization
import ckan.logic as logic
import ckan.lib.helpers as h
from ckanext.scheming.helpers import scheming_get_schema, scheming_field_by_name
import rdflib
from rdflib.plugins.sparql.results.tsvresults import TSVResultParser
from rdflib.term import Variable
from io import StringIO
from ckan.common import config


from ckanext.ids.dataspaceconnector.connector import Connector, ConnectorException
from ckanext.ids.metadatabroker.translations_broker_ckan import URI, \
    empty_result

# Define some shortcuts
# Ensure they are module-private so that they don't get loaded as available
# actions in the action API.
_validate = ckan.lib.navl.dictization_functions.validate
_table_dictize = ckan.lib.dictization.table_dictize
_check_access = logic.check_access
NotFound = logic.NotFound
NotAuthorized = logic.NotAuthorized
ValidationError = logic.ValidationError
_get_or_bust = logic.get_or_bust

# log = logging.getLogger('ckan.logic')
log = logging.getLogger("ckanext")

connector = Connector()
log.info("Using " + connector.broker_url + " as broker URL")

idsresource = rdflib.URIRef("https://w3id.org/idsa/core/Resource")

rdftype = rdflib.namespace.RDF["type"]


def _grouping_from_table_to_dict(triples_list: List[Dict],
                                 grouping_column: str = "s",
                                 key_column: str = "p",
                                 value_clumn: str = "o"):
    allgroups = set([x[grouping_column] for x in triples_list])
    allkeys = set([x[key_column] for x in triples_list])
    result = {x: {k: [] for k in allkeys}
              for x in allgroups}
    for t in triples_list:
        g = t[grouping_column]
        k = t[key_column]
        v = t[value_clumn]

        result[g][k].append(v)

    return result


def _parse_broker_tabular_response(raw_text):
    parser = TSVResultParser()
    input = StringIO(raw_text)

    result = parser.parse(source=input)
    return result


def _sparql_describe_many_resources(resources: Set[rdflib.URIRef]) -> str:
    query = """PREFIX owl: <http://www.w3.org/2002/07/owl#>
               SELECT ?s ?p ?o 
               WHERE { \n"""
    qparts = []
    for res in resources:
        qp = "{ ?s2 ?p ?o . \n"
        qp += "  ?s2 owl:sameAs " + res.n3() + " .\n"
        qp += "  BIND (URI(" + res.n3() + ") as ?s ) .}"
        qp += "UNION { " + res.n3() + " ?p ?o. "
        qp += "         BIND (URI(" + res.n3() + ") as ?s ) .}"
        qparts.append(qp)
    query += "\nUNION\n".join(qparts)
    query += "}"

    return query


# ToDo uncomment filter statement
def _sparl_get_all_resources(resource_type: str, fts_query: str, fq: list, facet_fields: list,
                             limit: int, offset: int, type_pred="https://www.trusts-data.eu/ontology/asset_type"):
    catalogiri = URI(config.get("ckanext.ids.connector_catalog_iri")).n3()
    #TODO: make this resource type specific
    dataset_schema = scheming_get_schema("dataset", "dataset", True)
    query = """
      PREFIX owl: <http://www.w3.org/2002/07/owl#>
      PREFIX ids: <https://w3id.org/idsa/core/>
      SELECT ?resultUri ?type ?title ?description ?assettype ?externalname ?license ?creationDate
      WHERE
      { ?resultUri a ?type . 
        ?conn <https://w3id.org/idsa/core/offeredResource> ?resultUri .
        ?resultUri ids:title ?title .
        ?resultUri ids:description ?description .
        ?resultUri owl:sameAs ?externalname .
        ?resultUri ids:standardLicense ?license .
        OPTIONAl {  ?resultUri ids:created ?creationDateTemp . }
        BIND (str(coalesce(?creationDateTemp, "Not Specified")) as ?creationDate)
        FILTER (!regex(str(?externalname),\"""" + config.get(
        'ckanext.ids.local_node_name') + """\",\"i\"))
        """
    facet_filters = build_facet_filters(fq, facet_fields, dataset_schema)
    for facet_filter in facet_filters:
        query += facet_filter

    if resource_type is None or resource_type == "None":
        query += "\n ?resultUri " + URI(type_pred).n3() + " ?assettype."
    else:
        typeuri = URI("https://www.trusts-data.eu/ontology/" + \
                      resource_type.capitalize())
        query += "\n ?resultUri " + URI(
            type_pred).n3() + " ?assettype ."
        query += "\nvalues ?assettype { " + typeuri.n3() + " } "
    if fts_query is not None:
        query += "FILTER regex(concat(?title, \" \",?description, \" \",str(?externalname)), \"" + fts_query + "\", \"i\")"
    query += "\n}"
    query += " LIMIT " + str(limit) + " OFFSET " + str(offset)
    return query


def build_facet_filters(fq: list, facet_fields: list, schema: dict):
    facets = dictionize_facet_query(fq, facet_fields, schema)
    facet_filters = []
    for facet in facets:
        facet_dict = facets[facet]
        facet_filter = "?resultUri <" + facet_dict["property"] + "> ?" + facet + ". values ?" + facet +" { " + " ".join(facet_dict["values"]) + " }"
        facet_filters.append(facet_filter)
    return facet_filters


def dictionize_facet_query(fq: list, facet_fields: list, schema: dict):
    facets = {}
    for facet in fq:
        facet_item = facet.split(":", 1)
        facet_key = facet_item[0]
        if facet_key in facet_fields:
            facet_value = sanitize_facet_value(facet_item[1])
            if facet_key in facets.keys():
                facets[facet_key]["values"].append(facet_value)
            else:
                scheming_field_by_name(schema["dataset_fields"], facet_key)
                facets[facet_key] = {
                    "property" : scheming_field_by_name(schema["dataset_fields"], facet_key)["display_property"],
                    "values" : [facet_value]
                }
    return facets


def sanitize_facet_value(value: str):
    result = value
    if "http" in value:
        result = value.lstrip('"').rstrip('"')
        result = "<" + result + ">"
    elif value.startswith('"'):
        result = value.replace('"', '')

    return result

def _sparl_get_facets(resource_type: str, fts_query: str, fq: str, facet_fields, type_pred="https://www.trusts-data.eu/ontology/asset_type"):
    catalogiri = URI(config.get("ckanext.ids.connector_catalog_iri")).n3()
    #TODO: make this resource type specific
    dataset_schema = scheming_get_schema("dataset", "dataset", True)
    facet_properties = []
    for facet_field in facet_fields:
        schema_field = scheming_field_by_name(dataset_schema["dataset_fields"], facet_field)
        if schema_field is not None:
            property = schema_field["display_property"]
            if "://" in property:
                facet_properties.append(URI(property).n3())
            else:
                facet_properties.append(property)


    query = """
          PREFIX  owl:  <http://www.w3.org/2002/07/owl#>
          PREFIX  ids:  <https://w3id.org/idsa/core/>
          SELECT  (count(?resultUri) as ?facet_count) (str(?facet) as ?facet_string) (str(?facet_value) as ?facet_value_string)
          WHERE
          { graph ?g
            {  
                ?conn     ids:offeredResource   ?resultUri .
                ?resultUri  owl:sameAs            ?externalname .
                FILTER (!regex(str(?externalname),\"""" + config.get('ckanext.ids.local_node_name') + """\",\"i\"))               
                OPTIONAL { ?resultUri ?facet ?facet_value }
                VALUES ?facet { """ + " ".join(facet_properties) + """ } """
    facet_filters = build_facet_filters(fq, facet_fields, dataset_schema)
    for facet_filter in facet_filters:
        query += facet_filter
    if resource_type is None or resource_type == "None":
        query += "\n ?resultUri " + URI(type_pred).n3() + " ?assettype."
    else:
        typeuri = URI("https://www.trusts-data.eu/ontology/" + \
                      resource_type.capitalize())
        query += "\n ?resultUri " + URI(
            type_pred).n3() + " ?assettype ."
        query += "\nvalues ?assettype { " + typeuri.n3() + " } "
    if fts_query is not None:
        query += "FILTER regex(concat(?title, \" \",?description, \" \",str(?externalname)), \"" + fts_query + "\", \"i\")"
    query += """ 
            }
          }
          group by ?facet ?facet_value
          order by ?facet DESC(?facet_count) ?facet_value
        """
    # query += " LIMIT " + str(limit) + " OFFSET " + str(offset)
    return query


def listofdicts2graph(lod: List[Dict],
                      s: str = "s",
                      p: str = "p",
                      o: str = "o"):
    g = rdflib.Graph()
    for ri in lod:
        s = URI(ri[s])
        p = URI(ri[p])
        if ri[o].startswith("<") and ri[o].endswith(">"):
            o = URI(ri[o])
        else:
            o = rdflib.Literal(ri[o])
        g.add((s, p, o))

    return g


def _to_ckan_package(raw_jsonld: Dict):
    package = dict()
    package['title'] = raw_jsonld['ids:title'][0]['@value']
    package['name'] = hashlib.sha256(
        raw_jsonld['@id'].encode('utf-8')).hexdigest()
    package['description'] = raw_jsonld['ids:description'][0]['@value']
    package['version'] = raw_jsonld['ids:version']
    package['theme'] = raw_jsonld['https://www.trusts-data.eu/ontology/theme'][
        '@id']
    package['type'] = get_resource_type(
        raw_jsonld['https://www.trusts-data.eu/ontology/asset_type']['@id'])
    package['owner_org'] = raw_jsonld['ids:publisher']['@id']
    return package


def get_resource_type(type):
    if type == "https://www.trusts-data.eu/ontology/Dataset":
        return "dataset"
    elif type == "https://www.trusts-data.eu/ontology/Application":
        return "appplication"
    elif type == "https://www.trusts-data.eu/ontology/Service":
        return "service"
    else:
        raise ValueError("Uknown dataset type: " + type + " Mapping failed.")


def graphs_to_artifacts(raw_jsonld: Dict):
    g = raw_jsonld["@graph"]
    artifact_graphs = [x for x in g if x["@type"] == "ids:Artifact"]
    return [x["sameAs"] for x in artifact_graphs]


def graphs_to_contracts(raw_jsonld: Dict,
                        broker_resource_uri: str):
    g = raw_jsonld["@graph"]
    contract_graphs = [x for x in g if x["@type"] == "ids:ContractOffer"]
    resource_graphs = [x for x in g if x["@type"] == "ids:Resource"]
    permission_graphs = [x for x in g if x["@type"] == "ids:Permission"]
    artifact_graphs = [x for x in g if x["@type"] == "ids:Artifact"]

    resource_uri = resource_graphs[0]["sameAs"]
    theirname = resource_uri
    organization_name = theirname.split("/")[2].split(":")[0]
    providing_base_url = "/".join(resource_uri.split("/")[:3])

    permission_graph_dict = {x["@id"]: x for x in permission_graphs}
    results = []
    for cg in contract_graphs:
        perms = cg["permission"]
        if not isinstance(perms, list):
            perms = [perms]
        r = dict()
        r["policies"] = [
            {"type": permission_graph_dict[per]["action"].upper().replace(
                "-", "_")} for per in perms]
        r["contract_start"] = cg["contractStart"]
        r["contract_end"] = cg["contractEnd"]
        r["title"] = clean_multilang(resource_graphs[0]["title"])
        r["errors"] = {}
        r["provider_url"] = providing_base_url
        r["resourceId"] = resource_uri
        r["artifactId"] = artifact_graphs[0]["sameAs"]
        r["artifactIds"] = [x["sameAs"] for x in artifact_graphs]
        r["contractId"] = cg["sameAs"]
        r["brokerResourceUri"] = broker_resource_uri

        results.append(r)

    return results


def rewrite_urls(provider_base, input_url):
    a = urlparse(input_url)
    return a._replace(netloc=provider_base).geturl()


# We pass the results of a query with
#    SELECT ?resultUri ?type ?title ?description ?assettype WHERE
def create_moot_ckan_result(binding):

    license = str(binding[Variable("license")])
    externalName = str(binding[Variable("externalname")])
    title = str(binding[Variable("title")])
    description = str(binding[Variable("description")])
    resultUri = str(binding[Variable("resultUri")])
    creationDate = str(binding[Variable("creationDate")])
    assetType = str(binding[Variable("assettype")])

    theirname = externalName
    organization_name = theirname.split("/")[2].split(":")[0]
    providing_base_url = "/".join(organization_name.split("/")[:3])
    organization_data = {
        "id": "52bc9332-2ba1-4c4f-bf85-5a141cd68423",
        "name": organization_name,
        "title": "Orga1",
        "type": "organization",
        "description": "",
        "image_url": "",
        "created": "2022-02-02T16:32:58.653424",
        "is_organization": True,
        "approval_status": "approved",
        "state": "active"
    }
    resources = []

    lictit = ""
    if "http://" or "https://" in license:
        lictit = license.split("/")[-1]

    packagemeta = deepcopy(empty_result)
    packagemeta["id"] = resultUri
    packagemeta["license_id"] = license + "LICENSE"
    packagemeta["license_url"] = license + "LICENSE"
    packagemeta["license_title"] = lictit
    packagemeta["metadata_created"] = creationDate
    packagemeta["metadata_modified"] = datetime.datetime.now().isoformat()
    packagemeta["name"] = title
    packagemeta["title"] = title
    packagemeta["description"] = description + "DESCRIPTION"
    packagemeta["type"] = assetType.split("/")[
        -1].lower()
    packagemeta["theme"] = "THEME"
    packagemeta["version"] = "VERSION"

    # These are the values we will use in succesive steps
    packagemeta["external_provider_name"] = organization_name
    packagemeta["to_process_external"] = config.get(
        "ckan.site_url") + "/ids/processExternal?uri=" + \
                                         urllib.parse.quote_plus(
                                             resultUri)
    packagemeta["provider_base_url"] = providing_base_url

    packagemeta["creator_user_id"] = "X"
    packagemeta["isopen"]: None
    packagemeta["maintainer"] = None
    packagemeta["maintainer_email"] = None
    packagemeta["notes"] = None
    packagemeta["num_tags"] = 0
    packagemeta["private"] = False
    packagemeta["state"] = "active"
    packagemeta["relationships_as_object"] = []
    packagemeta["relationships_as_subject"] = []
    packagemeta["url"] = providing_base_url
    packagemeta["tags"] = []  # (/)
    packagemeta["groups"] = []  # (/)

    packagemeta["dataset_count"] = 0
    packagemeta["service_count"] = 0
    packagemeta["application_count"] = 0
    packagemeta[packagemeta["type"] + "count"] = 1

    empty_ckan_resource = {
        "artifact": "http://artifact.uri/",
        "cache_last_updated": None,
        "cache_url": None,
        "created": datetime.datetime.now().isoformat(),
        "description": description,
        "format": "EXTERNAL",
        "hash": "SOMEHASH",
        "id": "http://artifact.uri/",
        "last_modified": datetime.datetime.now().isoformat(),
        "metadata_modified": datetime.datetime.now().isoformat(),
        "mimetype": "MEDIATYPE",
        "mimetype_inner": None,
        "name": title,
        "package_id": resultUri,
        "position": 0,
        "representation": "http://artifact.uri/",
        "resource_type": "resource",
        "size": 999,
        "state": "active",
        "url": "http://artifact.uri/",
        "url_type": "upload"
    }
    resources.append(empty_ckan_resource)

    packagemeta["organization"] = organization_data
    packagemeta["owner_org"] = organization_data["id"]
    packagemeta["resources"] = resources
    packagemeta["num_resources"] = 1
    # log.error(json.dumps(packagemeta,indent=1)+"\n.\n.\n.\n\n")

    return packagemeta


def clean_multilang(astring: str):
    if isinstance(astring, str):
        return astring
    if isinstance(astring, dict) and "@value" in astring.keys():
        return str(astring["@value"])
    return str(astring)


def graphs_to_ckan_result_format(raw_jsonld: Dict):
    g = raw_jsonld["@graph"]
    resource_graphs = [x for x in g if x["@type"] == "ids:Resource"]
    if "theme" not in resource_graphs[0].keys():
        return None
    representation_graphs = [x for x in g if
                             x["@type"] == "ids:Representation"]
    artifact_graphs = [x for x in g if x["@type"] == "ids:Artifact"]

    resource_uri = resource_graphs[0]["sameAs"]

    """
    print(10*"\n"+"~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~>>")
    print("\nresource_graphs = \n",json.dumps(resource_graphs,
                                            indent=1).replace("\n","\n\t"))
    print("\nartifact_graphs = \n", json.dumps(artifact_graphs,
                                             indent=1).replace("\n", "\n\t"))
    print("\nrepresentation_graphs = \n", json.dumps(representation_graphs,
                                             indent=1).replace("\n", "\n\t"))
    print("\n<<~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"+10 * "\n")
    """

    # ToDo get this from the central core as well
    theirname = resource_uri
    organization_name = theirname.split("/")[2].split(":")[0]
    providing_base_url = "/".join(organization_name.split("/")[:3])
    organization_data = {
        "id": "52bc9332-2ba1-4c4f-bf85-5a141cd68423",
        "name": organization_name,
        "title": "Orga1",
        "type": "organization",
        "description": "",
        "image_url": "",
        "created": "2022-02-02T16:32:58.653424",
        "is_organization": True,
        "approval_status": "approved",
        "state": "active"
    }
    resources = []

    packagemeta = deepcopy(empty_result)
    packagemeta["id"] = resource_uri
    packagemeta["license_id"] = resource_graphs[0][
        "standardLicense"] if "standardLicense" in resource_graphs[0] else None
    packagemeta["license_url"] = resource_graphs[0][
        "standardLicense"] if "standardLicense" in resource_graphs[0] else None
    packagemeta["license_title"] = resource_graphs[0][
        "standardLicense"] if "standardLicense" in resource_graphs[0] else None
    packagemeta["metadata_created"] = resource_graphs[0]["created"]
    packagemeta["metadata_modified"] = resource_graphs[0]["modified"]
    packagemeta["name"] = clean_multilang(resource_graphs[0]["title"])
    packagemeta["title"] = clean_multilang(resource_graphs[0]["title"])
    packagemeta["type"] = resource_graphs[0]["asset_type"].split("/")[
        -1].lower()
    packagemeta["theme"] = resource_graphs[0]["theme"].split("/")[-1]
    packagemeta["version"] = resource_graphs[0]["version"]

    # These are the values we will use in succesive steps
    packagemeta["external_provider_name"] = organization_name
    packagemeta["to_process_external"] = config.get(
        "ckan.site_url") + "/ids/processExternal?uri=" + \
                                         urllib.parse.quote_plus(
                                             resource_uri)
    packagemeta["provider_base_url"] = providing_base_url

    packagemeta["creator_user_id"] = "X"
    packagemeta["isopen"]: None
    packagemeta["maintainer"] = None
    packagemeta["maintainer_email"] = None
    packagemeta["notes"] = None
    packagemeta["num_tags"] = 0
    packagemeta["private"] = False
    packagemeta["state"] = "active"
    packagemeta["relationships_as_object"] = []
    packagemeta["relationships_as_subject"] = []
    packagemeta["url"] = providing_base_url
    packagemeta["tags"] = []  # (/)
    packagemeta["groups"] = []  # (/)

    packagemeta["dataset_count"] = 0
    packagemeta["service_count"] = 0
    packagemeta["application_count"] = 0
    packagemeta[packagemeta["type"] + "count"] = 1

    for rg in representation_graphs:
        artifact_this_res = [x for x in artifact_graphs
                             if x["@id"] == rg["instance"]][0]
        # logging.error(json.dumps(artifact_this_res,indent=1)+
        #              "\n~~~~~~~~~~~~~~~~~~~~~~~~~~~\n")
        empty_ckan_resource = {
            "artifact": artifact_this_res["@id"],
            "cache_last_updated": None,
            "cache_url": None,
            "created": resource_graphs[0]["created"],
            "description": clean_multilang(resource_graphs[0]["description"]),
            "format": "EXTERNAL",
            "hash": artifact_this_res["checkSum"],
            "id": rg["@id"],
            "last_modified": resource_graphs[0]["modified"],
            "metadata_modified": rg["modified"],
            "mimetype": rg["mediaType"],
            "mimetype_inner": None,
            "name": artifact_this_res["fileName"],
            "package_id": resource_graphs[0]["sameAs"],
            "position": 0,
            "representation": rg["sameAs"],
            "resource_type": "resource",
            "size": artifact_this_res["ids:byteSize"],
            "state": "active",
            "url": rg["sameAs"],
            "url_type": "upload"
        }
        resources.append(empty_ckan_resource)

    packagemeta["organization"] = organization_data
    packagemeta["owner_org"] = organization_data["id"]
    packagemeta["resources"] = resources
    packagemeta["num_resources"] = len(artifact_graphs)

    return packagemeta


@ckan.logic.side_effect_free
def broker_package_search(q=None, start_offset=0, limit=None, fq=None, facet_fields=None):
    log.debug("\n--- STARTING  BROKER SEARCH  ----------------------------\n")
    log.debug(str(q))
    log.debug(str(fq))
    log.debug("-----------------------------------------------------------\n")
    default_search = "*:*"
    search_string = None if q == default_search else q

    # By default we will search for all sorts of stuff
    requested_type = None
    try:
        if fq is not None:
            requested_type = [x for x in fq
                              if "+dataset_type" in x][0]
            requested_type = [x for x in requested_type.split(" ")
                              if x.startswith("+dataset_type")]
            requested_type = requested_type[0].split(":")[-1].capitalize()
    except:
        pass

    # log.debug("Requested search type was " + str(requested_type) + "\n\n")
    search_results = []


    if False:
    #if len(search_string) > 0 and search_string != default_search:
        raw_response = connector.search_broker(search_string=search_string,
                                               offset=start_offset, limit=limit)
        parsed_response = _parse_broker_tabular_response(raw_response)
        resource_uris = set([URI(x["resultUri"])
                             for x in parsed_response
                             if URI(x["type"]) == idsresource])
        descriptions = {
            ru.n3(): connector.ask_broker_for_description(ru.n3()[1:-1])
            for ru in resource_uris}

        for k, v in descriptions.items():
            pm = graphs_to_ckan_result_format(v)
            if pm is not None:
                search_results.append(pm)
    else:
        general_query = _sparl_get_all_resources(resource_type=requested_type, fts_query=search_string,
                                                 fq=fq, facet_fields=facet_fields, limit=limit, offset=start_offset)
        log.debug("Default search activated---- type:" + str(requested_type))
        # log.debug("QUERY :\n\t" + str(general_query).replace("\n", "\n\t"))

        try:
            raw_response = connector.query_broker(general_query)
            parsed_response = _parse_broker_tabular_response(raw_response)
            size_of_broker_results = len(parsed_response.bindings)
        except ConnectorException as e:
            log.debug(e.message)
            h.flash_error("It was not possbile to establish connection to the Broker. Please contact your administrator to investigate further.")
            size_of_broker_results = 0

        if size_of_broker_results > 0:
            log.debug(str(size_of_broker_results) + "   RESOURCES FOUND "
                                                "<------------------------------------\n")

        if size_of_broker_results == 0:
            return search_results

        for res in parsed_response.bindings:
            pm = create_moot_ckan_result(res)
            search_results.append(pm)

        facets_query = _sparl_get_facets(resource_type=requested_type, fts_query=search_string, fq=fq, facet_fields=facet_fields)
        facets_response = connector.query_broker(facets_query)

        parsed_facets_response = _parse_broker_tabular_response(facets_response)
        facets_result = refactor_facets_parsed_response(parsed_facets_response)

    # -- SLOW version
    # descriptions = {
    #     ru.n3(): connector.ask_broker_for_description(ru.n3()[1:-1])
    #     for ru in resource_uris}
    #
    # for k, v in descriptions.items():
    #     pm = graphs_to_ckan_result_format(v)
    #     if pm is not None:
    #         search_results.append(pm)

    # log.debug("---- END BROKER SEARCH ------------\n-----------\n----\n-----")
#    search_results["facets"] = facets
    response = {}
    response["results"] = search_results
    response["facets"] = facets_result
    #TODO: this should come from a different query, now it is just wrong
    response["count"] = len(search_results)
    response["search_facets"] = create_search_facets_object(facets_result)
    return response


def refactor_facets_parsed_response(facets_response):
    facets_result = {}
    dataset_schema = scheming_get_schema("dataset", "dataset", True)
    for facet_response in facets_response.bindings:
        facet_uri = str(facet_response[Variable("facet_string")])
        schema_field = scheming_field_by_display_property(dataset_schema["dataset_fields"], facet_uri)
        if schema_field is not None:
            result = {str(facet_response[Variable("facet_value_string")]):int(str(facet_response[Variable("facet_count")]))}
            field_name = schema_field.get("field_name")
            if field_name in facets_result:
                facets_result[field_name].update(result)
            else:
                facets_result[field_name] = result

    return facets_result


def create_search_facets_object(facets_result:dict):
    search_facets = {}
    for key in facets_result.keys():
        search_facets[key] = {'title': key}
        items = []
        for item_key in facets_result[key].keys():
            items.append({'name':item_key, 'display_name': item_key, 'count': int(facets_result[key][item_key])})
        search_facets[key].update({'items':items})
    return search_facets


def scheming_field_by_display_property(fields, display_property):
    """
    Simple helper to grab a field from a schema field list
    based on the display property passed. Returns None when not found.
    """
    for field in fields:
        try:
            if field.get("display_property") == display_property:
                return field
        except:
            pass


def strip_scheme(url: str):

    if "opendefinition" in url:
        pattern = r'^https?:\/\/www.'
    else:
        pattern = r'^https?:\/\/'
    return re.sub(pattern, '', url)



from ckanext.ids.dataspaceconnector.connector import Connector
from ckanext.ids.dataspaceconnector.offer import Offer
from ckanext.ids.dataspaceconnector.resource import Resource


def delete_from_dataspace_connector(data):
    local_resource_dataspace_connector = Connector().get_resource_api()
    offer = Offer(data)
    for value in data["resources"]:
        # this has also to run for every resource
        resource = Resource(value)
        if resource.representation_iri is not None:
            local_resource_dataspace_connector.delete_representation(resource)
        if resource.artifact_iri is not None:
            local_resource_dataspace_connector.delete_artifact(
                resource.artifact_iri, data={})
    if offer.offer_iri is not None:
        local_resource_dataspace_connector.delete_offered_resource(offer)



#
# Copyright 2020 Fraunhofer Institute for Software and Systems Engineering
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import requests
import json

# Suppress ssl verification warning
requests.packages.urllib3.disable_warnings()


class IdsApi:
    session = None
    recipient = None

    def __init__(self, recipient, auth=("admin", "password")):
        self.session = requests.Session()
        self.session.auth = auth
        self.session.verify = False

        self.recipient = recipient

    def descriptionRequest(self, recipient, elementId):
        url = self.recipient + "/api/ids/description"
        params = {}
        if recipient is not None:
            params["recipient"] = recipient
        if elementId is not None:
            params["elementId"] = elementId

        response = self.session.post(url, params=params)
        return json.loads(response.text)

    def contractRequest(self, recipient, resourceId, artifactId, download, contract):
        url = self.recipient + "/api/ids/contract"
        params = {}
        if recipient is not None:
            params["recipient"] = recipient
        if resourceId is not None:
            params["resourceIds"] = resourceId
        if artifactId is not None:
            params["artifactIds"] = artifactId
        if download is not None:
            params["download"] = download

        response = self.session.post(
            url, params=params, json=self.toListIfNeeded(contract)
        )
        return json.loads(response.text)

    def toListIfNeeded(self, obj):
        if isinstance(obj, list):
            return obj
        else:
            return [obj]

    def create_data_source(self, data={}):
        response = self.session.post(self.recipient + "/api/datasources", json=data)
        return json.loads(response.text)

    def create_endpoint(self, data={}):
        response = self.session.post(self.recipient + "/api/endpoints", json=data)
        return json.loads(response.text)

    def link_endpoint_datasource(self, endpointId, datasourceId):
        response = self.session.put(self.recipient + "/api/endpoints/" + endpointId + "/datasource/" + datasourceId)
        return response.status_code == 204

    def create_route(self, data={}):
        response = self.session.post(self.recipient + "/api/routes/", json=data)
        return json.loads(response.text)

    def link_route_endpoint(self, routeId, endpointId):
        response = self.session.put(self.recipient + "/api/routes/" + routeId + "/endpoint/end", data=endpointId)
        return response.status_code == 204


import datetime
import json
import logging
import time
from os.path import join as pathjoin

import requests
from ckan.common import config
from requests.auth import HTTPBasicAuth

from ckanext.ids.dataspaceconnector.resourceapi import ResourceApi

log = logging.getLogger("ckanext")


class ConnectorException(Exception):
    def __init__(self, m):
        self.message = m

    def __str__(self):
        return "CONNECTOR_EXCEPTION " + self.message


class Connector:
    url = None
    auth = ()

    # def __init__(self, url, username, password):
    #     self.url = url
    #     self.auth = (username, password)
    #     self.broker_url = config.get('ckanext.ids.trusts_central_broker',
    #                                  'http://central-core:8282/infrastructure')
    #     self.broker_url = 'http://central-core:8080/infrastructure'

    def __init__(self):
        port = config.get('ckanext.ids.trusts_local_dataspace_connector_port',
                          '8080')
        self.url = config.get(
            'ckanext.ids.trusts_local_dataspace_connector_url',
            'provider-core') + ":" + str(port)
        self.auth = (
            config.get(
                'ckanext.ids.trusts_local_dataspace_connector_username'),
            config.get(
                'ckanext.ids.trusts_local_dataspace_connector_password'))
        # ToDo Make sure this config is working
        self.broker_url = config.get('ckanext.ids.trusts_central_broker',
                                     'http://central-core:8282/infrastructure')
        # self.broker_url = 'http://central-core:8080/infrastructure'
        self.broker_knows_us_timestamp = None
        self.broker_knows_us_limit = 10
        self.my_catalog_ids = []
        self.resourceAPI = ResourceApi(self.url, self.auth)

    def broker_knows_us(self):
        if self.broker_knows_us_timestamp is None:
            return False
        tnow = datetime.datetime.now()
        dt = tnow - self.broker_knows_us_timestamp
        if dt.total_seconds() > self.broker_knows_us_limit:
            return False
        log.debug("\n________ BROKER STILLS KNOWS US _______________")
        return True

    def get_resource_api(self):
        return self.resourceAPI

    def search_broker(self, search_string: str,
                      limit: int = 100,
                      offset: int = 0):
        self.announce_to_broker()
        params = {"recipient": self.broker_url,
                  "limit": limit,
                  "offset": offset}
        headers = {"Content-type": "application/json",
                   "accept": "*/*"}

        url = pathjoin(self.url, "api/ids/search")
        data = search_string.encode("utf-8")
        response = requests.post(url=url,
                                 params=params,
                                 data=data,
                                 auth=HTTPBasicAuth(self.auth[0],
                                                    self.auth[1]))
        if response.status_code > 299 or response.text is None:
            log.error("Got code " + str(response.status_code) + " in search")
            log.error("Response Text: " + str(response.text))
            log.error("Provided Data: " + data.decode("utf-8"))
            log.error("URL: " + url)
            log.error("PARAMS: " + json.dumps(params, indent=2))

            raise ConnectorException("Code: " + str(response.status_code) +
                                     " Text: " + str(response.text))

        return response.text

    def query_broker(self, query_string: str, return_if_417=False):
        if not return_if_417:
            self.announce_to_broker()
        params = {"recipient": self.broker_url}
        url = pathjoin(self.url, "api/ids/query")
        data = query_string.encode("utf-8")

        response = requests.post(url=url,
                                 params=params,
                                 data=data,
                                 auth=HTTPBasicAuth(self.auth[0],
                                                    self.auth[1]))
        if return_if_417:
            return response
        if response.status_code > 299 or response.text is None:
            log.error("Got code " + str(response.status_code) + " in search")
            log.error("Provided Data: " + data.decode("utf-8"))
            raise ConnectorException("Code: " + str(response.status_code) +
                                     " Text: " + str(response.text))

        return response.text

    def ask_broker_for_description(self, element_uri: str):
        self.announce_to_broker()
        resource_contract_tuples = []

        if len(element_uri) < 5 or ":" not in element_uri:
            return {}
        params = {"recipient": self.broker_url,
                  "elementId": element_uri}
        url = pathjoin(self.url, "api/ids/description")
        response = requests.post(url=url,
                                 params=params,
                                 auth=HTTPBasicAuth(self.auth[0],
                                                    self.auth[1]))
        if response.status_code > 299 or response.text is None:
            log.error("Got code " + str(response.status_code) + " in describe")
            raise ConnectorException("Code: " + str(response.status_code) +
                                     " Text: " + str(response.text))

        graphs = response.json()
        return graphs

    def fetch_catalog_ids(self):
        rj = self.resourceAPI.get_catalogs()
        self.my_catalog_ids = [x["_links"]["self"]["href"]
                               for x in rj["_embedded"]["catalogs"]]
        return self.my_catalog_ids

    def _build_query_my_resources(self):
        q = """
            PREFIX owl: <http://www.w3.org/2002/07/owl#>
            SELECT ?resultUri ?conn { GRAPH ?g  { 
            """
        qparts = []
        for cat in self.my_catalog_ids:
            qpart = """
              {
              ?resultUri a <https://w3id.org/idsa/core/Resource> .
              ?conn <https://w3id.org/idsa/core/offeredResource> ?resultUri .
              ?conn owl:sameAs <""" + cat + ">. "
            qpart += "}"
            qparts.append(qpart)

        q += "\nUNION\n".join(qparts)
        q += "} }"
        return q

    def announce_to_broker(self, force=False):
        # If for some reason this is the first resource we send to the
        # broker, we have to first register this connector.
        # We check the broker response, because if we register when
        # there is already an index for this connector, all resources
        # are deleted :S
        log.debug("----ANOUNCE BROKER\n\n")
        if self.broker_knows_us() and not force:
            log.debug("\t|---BROKER KNOWS US 1")
            return True

        self.fetch_catalog_ids()
        q = self._build_query_my_resources()
        r = self.query_broker(q, return_if_417=True)

        self.broker_knows_us_timestamp = datetime.datetime.now()

        need_to_announce = force
        if force:
            log.debug("\t|--->  FORCING ANNOUNCE")
        # If the Index is empty, the broker is probably fresh restarted
        if r.status_code == 417:
            #    and "empty" in str(r.json()).lower():
            need_to_announce = True
            log.error("\t|--- 417")

        # If there are no records for this connector, it doesn't harm to
        # announce ourselves again
        numlines = -1
        if r.status_code < 299:
            numlines = len([x for x in r.text.split("\n")])
            if numlines < 1:  # It should contain at least the header line
                need_to_announce = True
                log.error("\t|--- 200 but no lines")

        log.debug("\t|---NEED" + str(need_to_announce))
        if need_to_announce:
            params = {"recipient": self.broker_url}
            url = pathjoin(self.url, "api/ids/connector/update")
            response = requests.post(url=url,
                                     params=params,
                                     auth=HTTPBasicAuth(self.auth[0],
                                                        self.auth[1]))
            log.debug("\t|--- /connector/update response was " + str(
                response.status_code))
            if response.status_code < 299:
                self.broker_knows_us_timestamp = datetime.datetime.now()
        else:
            log.debug("\t---____ NO NEED TO ANNOUNCE US _______________")
            log.debug("\t---" + str(r.status_code) + "  with lines: " + str(
                numlines))
            self.broker_knows_us_timestamp = datetime.datetime.now()

        return True

    def send_resource_to_broker(self, resource_uri: str):
        sent_succesfully = False
        attempts = 0
        while not sent_succesfully:
            attempts += 1
            sent_succesfully = True
            self.announce_to_broker()
            params = {"recipient": self.broker_url,
                      "resourceId": resource_uri}
            url = pathjoin(self.url, "api/ids/resource/update")
            response = requests.post(url=url,
                                     params=params,
                                     auth=HTTPBasicAuth(self.auth[0],
                                                        self.auth[1]))
            log.debug("------ \n\n REQUEST TO ADD TO BROKER " + url + " :")
            log.debug(json.dumps(params, indent=1))
            log.debug("------ RESPONSE OF SEND RESOURCE TO BROKER IS :  ")
            log.debug(response.text)
            log.debug("\n\n\n\n ----------------------\n -----------------\n")
            if (response.status_code > 299 or ("RejectionMessage" in
                                               response.text)):
                sent_succesfully = False
                log.error("\n\n----- THE BROKER REJECTED THE ASSET " +str(
                    attempts)+" TIMES")
                log.error("Connector returned: " + str(response.status_code))
                log.error("Something went wrong when sending announcment to "
                          "broker, retrying in 1s")
                log.debug("\n| Will first force announce")
                self.announce_to_broker(force=True)
                log.debug("----------------\n|\n|\n")
                time.sleep(1)

            if attempts > 3:
                log.error("10 attempts not push to broker failed")
                return False

        return response.status_code < 299



import logging

import ckan.plugins.toolkit as toolkit

import pathlib
from ckanext.ids.model import IdsResource, IdsAgreement, IdsSubscription
from urllib.parse import urlparse
from ckanext.ids.dataspaceconnector.resourceapi import ResourceApi
from ckanext.ids.dataspaceconnector.subscriptionapi import SubscriptionApi
from ckanext.ids.dataspaceconnector.idsapi import IdsApi

log = logging.getLogger("ckanext.ids.dsc.subscribe")

consumerUrl = toolkit.config.get('ckanext.ids.trusts_local_dataspace_connector_url') + ":" + toolkit.config.get('ckanext.ids.trusts_local_dataspace_connector_port')
username = toolkit.config.get('ckanext.ids.trusts_local_dataspace_connector_username')
password = toolkit.config.get('ckanext.ids.trusts_local_dataspace_connector_password')
consumer_alias = consumerUrl

consumer = IdsApi(consumerUrl, auth=(username, password))
local_node = toolkit.config.get("ckanext.ids.trusts_local_dataspace_connector_url")


class Subscription:
    offer_url = None
    provider_alias = None
    agreement_url = None
    first_artifact = None
    remote_artifact = None
    data_source = None
    endpoint = None
    route = None


    def __init__(self, offer_url, agreement, user_email):
        self.offer_url = offer_url
        self.agreement = agreement
        self.user_email = user_email

        parsed_url = urlparse(offer_url)
        self.provider_alias = parsed_url.scheme + "://" + parsed_url.netloc


# Consumer

    def subscribe(self):
       # self.create_data_source()
       # log.debug("creating endpoint...")
       # self.create_endpoint()
       # log.debug("endpoint created!")
       #  self.link_endpoint_data_source()
       # log.debug("creating route...")
       # self.create_route()
       # log.debug("route created!")
       # log.debug("linking endpoint to route...")
       # self.link_endpoint_route()
       # log.debug("linking completed!")
        log.debug("getting local agreement")
        local_agreement = self.agreement
        log.debug("agreement retrieved!")
        log.debug("getting request")
        consumerResources = ResourceApi(consumerUrl)
        artifacts = consumerResources.get_artifacts_for_agreement(local_agreement.id)
        first_artifact = artifacts["_embedded"]["artifacts"][0]["_links"]["self"]["href"]
        representations = consumerResources.get_representations_for_artifact(first_artifact)
        first_representation = representations["_embedded"]["representations"][0]["_links"]["self"]["href"]
        requests = consumerResources.get_requests_for_representation(first_representation)
        first_request = requests["_embedded"]["resources"][0]["_links"]["self"]["href"]
        log.debug("Request retrieved!")
        # subscribe to the requested artifact
        log.debug("Creating subscription client...")
        consumerSub = SubscriptionApi(consumerUrl)
        log.debug("Client created!")
        data = {
            "title": "CKAN on " + local_node + " asset subscription: " + self.offer_url,
            "description": first_request,
            "target": first_request,
            "location": "http://note-service:5055/notify?ids-toemail=" + self.user_email,
            "subscriber":  "http://note-service:5055/notify?ids-toemail=" + self.user_email,
            "pushData": "false",
        }
        log.debug(data)
        log.debug("creating non-IDS subscription...")
        response = consumerSub.create_subscription(data=data)
        log.debug("non-IDS subscription created!")
        log.debug("Persisting subscription...")
        local_subscription = IdsSubscription(id=response, agreement=local_agreement, user=username)
        local_subscription.save()
        log.debug("Subscription persisted!")
        log.debug(response)

        ## this is used to create ids subscription
        ## subscribe to the remote offer
        log.debug("Creating IDS subscription...")
        data = {
            "title": "DSC on " + local_node + "asset subscription: " + self.offer_url,
            "description": self.offer_url,
            "target": self.offer_url,
            "location": consumerUrl + "/api/ids/data",
            "subscriber": consumerUrl,
            "pushData": "true",
        }
        log.debug(data)
        response = consumerSub.subscription_message(
            data=data, params={"recipient": self.provider_alias + "/api/ids/data"}
        )
        log.debug(response.status_code)
        log.debug(response.text)
        if response.status_code == 200:
            print(response.text)
            return True
        else:
            return False


    def create_data_source(self):
        data = {
            "authentication": {
                "key": "dsc_user_read",
                "value": toolkit.config.get('ckanext.ids.trusts_local_dataspace_connector_password')
            },
            "type": "REST"
        }
        data_source = consumer.create_data_source(data)
        self.data_source = data_source["id"]
        return data_source

    def create_endpoint(self):
        data = {
            "location": "http://note-service:5055/notify?ids-toemail=" + self.user_email,
            "type": "GENERIC"
        }
        endpoint = consumer.create_endpoint(data)
        self.endpoint = pathlib.PurePath(endpoint["_links"]["self"]["href"]).name
        return endpoint

    def link_endpoint_data_source(self):
        response = consumer.link_endpoint_datasource(self.endpoint, self.data_source)
        return response

    def create_route(self):
        data = {
            "title": "Notification service Update Route",
            "deploy": "Camel"
        }
        response = consumer.create_route(data)
        self.route = response["_links"]["self"]["href"]
        return response

    def link_endpoint_route(self):
        response = consumer.link_route_endpoint(pathlib.PurePath(self.route).name, self.endpoint)
        return response


import datetime
import json
import logging

import ckan.lib.helpers as h
import pytz
from ckan.common import _, config


class Contract:
    pkg_id: None
    contract_start = None
    contract_end = None
    contract_iri: None
    title: None
    rule_iris: []
    errors: {}
    policies: []

    def __init__(self, contract_data):
        """
        contract_data is a dict that must contain the following keys:
         `pkg_id` : the id of the ckan package
         `title`  : the title of the contract
         `contract_start_date`  : date of start of contract  iso
         `contract_start_time`  : date of start of contract  iso format
         `contract_start_tz`    : timezone of start, recongizable by pytz
         `contract_end_date`  : date of end of contract  iso
         `contract_end_time`  : date of end of contract  iso format
         `contract_end_tz`    : timezone of end, recongizable by pytz
         `{POLICY_TYPE}'   : the key itself should be one of the types in
                             the `ckanext/ids/usage_control.json` file

        """
        #print("\n\n---------------------------------")
        #print(json.dumps(contract_data,indent=1))
        #print("\n---------------------------------")
        if "pkg_id" in contract_data:
            self.pkg_id = contract_data["pkg_id"]
        else:
            print("No pkg_id was founnd... why?")
            raise KeyError
        if "title" in contract_data:
            self.title = contract_data["title"]
        else:
            self.title = None
        self.errors = {
                        "contract_start_date": [],
                        "contract_start_time": [],
                        "contract_start_tz": [],
                        "contract_end_date": [],
                        "contract_end_time": [],
                        "contract_end_tz": [],
                        }
        self.contract_start = self.validate_date_inputs("contract_start",
                                                        contract_data,
                                                        self.errors)
        print("validated start date")
        self.contract_end = self.validate_date_inputs("contract_end",
                                                      contract_data,
                                                      self.errors)
        print("validated end date")
        self.policies = self.validate_policies(contract_data, self.errors)
        self.errors = {key:val
                       for key, val
                       in self.errors.items()
                       if len(val)}

    def converter(self, o):
        if isinstance(o, datetime.datetime):
            return o.isoformat()
        else:
            return o.__dict__

    def toJSON(self):
        return json.dumps(self, default=self.converter,
                      sort_keys=True, indent=4)

    def validate_policies(self, data, errors):
        policies_schema = config.get("ckanext.ids.usage_control_policies")
        policies = []
        for policy_template in policies_schema["policy_templates"]:
            policy_type = policy_template["type"]
            if data.get(policy_type) is None:
                continue
            policy = {
                "type": policy_type
            }
            for field in policy_template["fields"]:
                errors[field["field_name"]] = []
                original_field_name = field["field_name"].split(policy_type + "_", 1)[1]
                try:
                    value = data.get(field["field_name"])
                    if value is None:
                        raise ValueError
                    policy[original_field_name] = value
                except Exception:
                    error_message = policy_type + " was selected, " + original_field_name + " is required."
                    errors[field["field_name"]].append(error_message)
            policies.append(policy)
        if len(policies) == 0:
            errors["policies"] = ["None of the policies was selected, please choose one of the available."]
        return policies

    def validate_date_inputs(self, key, data, errors):
        date_error = _('Date format incorrect')
        time_error = _('Time format incorrect')

        date = None

        def get_input(suffix):
            inpt = key + '_' + suffix
            key_value = data.get(inpt)

            return inpt, key_value

        date_key, value = get_input('date')
        value_full = ''

        if value:
            try:
                value_full = value
                date = h.date_str_to_datetime(value)
            except (TypeError, ValueError) as e:
                errors[date_key].append(date_error)
        else:
            errors[date_key].append(_('Date is required.'))

        time_key, value = get_input('time')
        if value:
            if not value_full:
                errors[date_key].append(
                    _('Date is required when a time is provided'))
            else:
                try:
                    value_full += 'T' + value
                    date = h.date_str_to_datetime(value_full)
                except (TypeError, ValueError) as e:
                    errors[time_key].append(time_error)
        else:
            errors[time_key].append(_('Time is required.'))

        tz_key, value = get_input('tz')
        if value:
            if value not in pytz.all_timezones:
                errors[tz_key].append('Invalid timezone')
            else:
                if isinstance(date, datetime.datetime):
                    date = pytz.timezone(value).localize(date)
        else:
            errors[tz_key].append(_('Timezone is required.'))
        return date



import rdflib

def URI(somestr: str):
    if isinstance(somestr, rdflib.URIRef):
        return somestr
    if somestr.startswith("<"):
        somestr = somestr[1:]
    if somestr.endswith(">"):
        somestr = somestr[:-1]
    return rdflib.URIRef(somestr)


empty_result = {
    "author": None,
    "author_email": None,
    "creator_user_id": "__MISSING__", #  (/)
    "id": "__MISSING__",   # (/)
    "isopen": None,    # (/)
    "license_id": "__MISSING__",  # (/)
    "license_title": "__MISSING__",  # (/)
    "license_url": "__MISSING__",   # (/)
    "maintainer": None, # (/)
    "maintainer_email": None,  # (/)
    "metadata_created": "__MISSING__",   # (/)
    "metadata_modified": "__MISSING__",   # (/)
    "name": "__MISSING__", # (/)
    "notes": None,  # (/)
    "num_resources": 0,  # (/)
    "num_tags": 0, # (/)
    "owner_org": "__MISSING__",   # (/)
    "private": None, # (/)
    "state": "active", # (/)
    "theme": "__MISSING__",   # (/)
    "title": "__MISSING__",   # (/)
    "type": "__MISSING__",   # (/)
    "url": None,  # (/)
    "version": "__MISSING__",  # (/)
    "tags": [],  # (/)
    "groups": [],  # (/)
    "dataset_count": 0,   # (/)
    "service_count": 0,  # (/)
    "application_count": 0,  # (/)
    "relationships_as_object": [], # (/)
    "relationships_as_subject": [], # (/)
    "resources": [ ],
    "organization": {}  # (/)
}

empty_organization = {
  "id": "52bc9332-2ba1-4c4f-bf85-5a141cd68423",
  "name": "orga1",
  "title": "Orga1",
  "type": "organization",
  "description": "",
  "image_url": "",
  "created": "2022-02-02T16:32:58.653424",
  "is_organization": True,
  "approval_status": "approved",
  "state": "active"
}

empty_resource = {
            "artifact":  "__MISSING__",
            "cache_last_updated": None,
            "cache_url": None,
            "created":  "__MISSING__",
            "description": "<https://w3id.org/idsa/core/description>",
            "format":  "__MISSING__",
            "hash": "",
            "id":  "__MISSING__",
            "last_modified":  "<https://w3id.org/idsa/core/modified>",
            "metadata_modified":  "<https://w3id.org/idsa/core/modified>",
            "mimetype":  "__MISSING__",
            "mimetype_inner": None,
            "name":  "__MISSING__",
            "package_id":  "__MISSING__",
            "position": 0,
            "representation":  "<https://w3id.org/idsa/core/representation>",
            "resource_type": "resource",
            "size":  "__MISSING__",
            "state": "active",
            "url": "__MISSING__",
            "url_type": "upload"
        }

translation_ckanFromBroker_result = {
    "maintainer" : URI("<https://w3id.org/idsa/core/publisher>"),
    "title": URI("<https://w3id.org/idsa/core/title>"),
    "version": URI("<https://w3id.org/idsa/core/version>"),
    "owner_org": URI("<https://w3id.org/idsa/core/sovereign>"),
    "name":URI("<https://w3id.org/idsa/core/title>"),
    "metadata_modified":URI("<https://w3id.org/idsa/core/modified>"),
    "metadata_created": URI("<https://w3id.org/idsa/core/created>"),
    "creator_user_id": URI("<https://w3id.org/idsa/core/sovereign>")
}


from ckan import model
import ckan.plugins.toolkit as toolkit


def create_pushed_to_dataspace_connector_activity(context, pkg_id):
    """
    Log a 'pushed to dataspace connector' activity in the activity stream
    """
    user = context['user']
    user_id = None
    user_by_name = model.User.by_name(user)
    if user_by_name is not None:
        user_id = user_by_name.id

    package = toolkit.get_action('package_show')(context.copy(), {'id': pkg_id})

    activity_dict = {
        'activity_type': 'pushed to dataspace connector',
        'user_id': user_id,
        'object_id': pkg_id,
        'data':
            {'package': package}
    }

    activity_create_context = {
        'model': model,
        'user': user_id or user,
        'defer_commit': False,
        'ignore_auth': True,
    }

    create_activity = toolkit.get_action('activity_create')
    create_activity(activity_create_context, activity_dict)


def create_created_contract_activity(context, pkg_id):
    """
    Log a 'pushed to dataspace connector' activity in the activity stream
    """
    user = context['user']
    user_id = None
    user_by_name = model.User.by_name(user)
    if user_by_name is not None:
        user_id = user_by_name.id
    package = toolkit.get_action('package_show')(context.copy(), {'id': pkg_id})
    activity_dict = {
        'activity_type': 'created contract',
        'user_id': user_id,
        'object_id': pkg_id,
        'data':
            {'package': package}
    }
    activity_create_context = {
        'model': model,
        'user': user_id or user,
        'defer_commit': False,
        'ignore_auth': True,
    }
    create_activity = toolkit.get_action('activity_create')
    create_activity(activity_create_context, activity_dict)
